cimport ruptures.detection._detection.ekcpd as ekcpd

from libc.stdlib cimport malloc, free

cimport cython
import numpy as np

cpdef ekcpd_L2(double[:,:] signal, int n_bkps, int min_size):

    # Allocate and initialize kernel description
    cdef ekcpd.KernelLinear kernelLinearDesc
    cdef ekcpd.KernelGeneric kernelDesc
    kernelDesc.name = LINEAR_KERNEL_NAME
    kernelLinearDesc.pBaseObj = &kernelDesc

    return ekcpd_core(signal, n_bkps, min_size, &kernelLinearDesc)


cpdef ekcpd_pelt_L2(double[:,:] signal, double beta, int min_size):

    # Allocate and initialize kernel description
    cdef ekcpd.KernelLinear kernelLinearDesc
    cdef ekcpd.KernelGeneric kernelDesc
    kernelDesc.name = LINEAR_KERNEL_NAME
    kernelLinearDesc.pBaseObj = &kernelDesc

    return ekcpd_pelt_core(signal, beta, min_size, &kernelLinearDesc)


cpdef ekcpd_Gaussian(double[:,:] signal, int n_bkps, int min_size, double gamma):

    # Allocate and initialize kernel description
    cdef ekcpd.KernelGaussian kernelGaussianDesc
    cdef ekcpd.KernelGeneric kernelDesc
    kernelDesc.name = GAUSSIAN_KERNEL_NAME
    kernelGaussianDesc.pBaseObj = &kernelDesc
    kernelGaussianDesc.gamma = gamma

    return ekcpd_core(signal, n_bkps, min_size, &kernelGaussianDesc)


cpdef ekcpd_pelt_Gaussian(double[:,:] signal, double beta, int min_size, double gamma):

    # Allocate and initialize kernel description
    cdef ekcpd.KernelGaussian kernelGaussianDesc
    cdef ekcpd.KernelGeneric kernelDesc
    kernelDesc.name = GAUSSIAN_KERNEL_NAME
    kernelGaussianDesc.pBaseObj = &kernelDesc
    kernelGaussianDesc.gamma = gamma

    return ekcpd_pelt_core(signal, beta, min_size, &kernelGaussianDesc)


cpdef ekcpd_cosine(double[:,:] signal, int n_bkps, int min_size):

    # Allocate and initialize kernel description
    cdef ekcpd.KernelCosine kernelCosineDesc
    cdef ekcpd.KernelGeneric kernelDesc
    kernelDesc.name = COSINE_KERNEL_NAME
    kernelCosineDesc.pBaseObj = &kernelDesc

    return ekcpd_core(signal, n_bkps, min_size, &kernelCosineDesc)


cpdef ekcpd_pelt_cosine(double[:,:] signal, double beta, int min_size):

    # Allocate and initialize kernel description
    cdef ekcpd.KernelLinear kernelCosineDesc
    cdef ekcpd.KernelGeneric kernelDesc
    kernelDesc.name = COSINE_KERNEL_NAME
    kernelCosineDesc.pBaseObj = &kernelDesc

    return ekcpd_pelt_core(signal, beta, min_size, &kernelCosineDesc)


cdef ekcpd_core(double[:,:] signal, int n_bkps, int min_size, void *kernelDescObj):
    cdef:
        int n_samples = signal.shape[0]
        int n_dims = signal.shape[1]

    # Allocate and initialize structure for result of c function
    cdef int[::1] path_matrix_flat = np.empty((n_bkps+1)*(n_samples+1), dtype=np.dtype("i"))
    # Make it C compatible in terms of memory contiguousness
    cdef double[:, ::1] signal_arr = np.ascontiguousarray(signal)
    try:
        ekcpd.ekcpd_compute(&signal_arr[0, 0], n_samples, n_dims, n_bkps, min_size, kernelDescObj, &path_matrix_flat[0])
    except:
        print("An exception occurred.")

    return np.asarray(path_matrix_flat)


cdef ekcpd_pelt_core(double[:,:] signal, double beta, int min_size, void *kernelDescObj):
    cdef:
        int n_samples = signal.shape[0]
        int n_dims = signal.shape[1]

    # Allocate and initialize structure for result of c function
    cdef int[::1] path_matrix_flat = np.empty((n_samples+1), dtype=np.dtype("i"))
    # Make it C compatible in terms of memory contiguousness
    cdef double[:, ::1] signal_arr = np.ascontiguousarray(signal)
    try:
        ekcpd.ekcpd_pelt_compute(&signal_arr[0, 0], n_samples, n_dims, beta, min_size, kernelDescObj, &path_matrix_flat[0])
    except:
        print("An exception occurred.")

    return np.asarray(path_matrix_flat)


r"""CostL2 (least squared deviation)"""
from ruptures.costs import NotEnoughPoints

from ruptures.base import BaseCost


class CostL2(BaseCost):

    r"""
    Least squared deviation.
    """

    model = "l2"

    def __init__(self):
        """Initialize the object."""
        self.signal = None
        self.min_size = 2

    def fit(self, signal) -> "CostL2":
        """Set parameters of the instance.

        Args:
            signal (array): array of shape (n_samples,) or (n_samples, n_features)

        Returns:
            self
        """
        if signal.ndim == 1:
            self.signal = signal.reshape(-1, 1)
        else:
            self.signal = signal

        return self

    def error(self, start, end) -> float:
        """Return the approximation cost on the segment [start:end].

        Args:
            start (int): start of the segment
            end (int): end of the segment

        Returns:
            segment cost

        Raises:
            NotEnoughPoints: when the segment is too short (less than `min_size` samples).
        """
        if end - start < self.min_size:
            raise NotEnoughPoints

        return self.signal[start:end].var(axis=0).sum() * (end - start)



print("import: 'ruptures'")
import ruptures

print("import: 'ruptures.costs'")
import ruptures.costs

print("import: 'ruptures.datasets'")
import ruptures.datasets

print("import: 'ruptures.detection'")
import ruptures.detection

print("import: 'ruptures.metrics'")
import ruptures.metrics

print("import: 'ruptures.show'")
import ruptures.show

print("import: 'ruptures.utils'")
import ruptures.utils




"""Offline change point detection for Python."""

from .datasets import pw_constant, pw_linear, pw_normal, pw_wavy
from .detection import Binseg, BottomUp, Dynp, KernelCPD, Pelt, Window
from .exceptions import NotEnoughPoints
from .show import display

# Convenient access to the version number
from .version import version as __version__



r"""Rand index (`randindex`)"""
from ruptures.metrics import hamming


def randindex(bkps1, bkps2):
    """Rand index for two partitions. The result is scaled to be within 0 and
    1.

    Args:
        bkps1 (list): list of the last index of each regime.
        bkps2 (list): list of the last index of each regime.

    Returns:
        float: Rand index
    """
    return 1 - hamming(bkps1, bkps2)
